@@ -19,8 +19,7 @@
    Authors: Andy Hertzfeld <andy@eazel.com>
  */
 
-#ifndef EEL_GRAPHIC_EFFECTS_H
-#define EEL_GRAPHIC_EFFECTS_H
+#pragma once
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdk.h>
@@ -30,6 +29,4 @@ GdkPixbuf *eel_create_spotlight_pixbuf (GdkPixbuf *source_pixbuf);
 
 /* return a pixbuf colorized with the color specified by the parameters */
 GdkPixbuf* eel_create_colorized_pixbuf (GdkPixbuf *source_pixbuf,
-					GdkRGBA *color);
-
-#endif /* EEL_GRAPHIC_EFFECTS_H */
+                                        GdkPixbuf *dest);
