@@ -20,18 +20,16 @@
 
 #pragma once
 
-#include <gdk/gdk.h>
-#include <gio/gio.h>
-#include <gtk/gtk.h>
+#include <adwaita.h>
 
 #include "nautilus-types.h"
 
 G_BEGIN_DECLS
 #define NAUTILUS_TYPE_APPLICATION (nautilus_application_get_type())
-G_DECLARE_DERIVABLE_TYPE (NautilusApplication, nautilus_application, NAUTILUS, APPLICATION, GtkApplication)
+G_DECLARE_DERIVABLE_TYPE (NautilusApplication, nautilus_application, NAUTILUS, APPLICATION, AdwApplication)
 
 struct _NautilusApplicationClass {
-	GtkApplicationClass parent_class;
+	AdwApplicationClass parent_class;
 
         void  (*open_location_full) (NautilusApplication     *application,
                                      GFile                   *location,
@@ -43,8 +41,7 @@ struct _NautilusApplicationClass {
 
 NautilusApplication * nautilus_application_new (void);
 
-NautilusWindow *     nautilus_application_create_window (NautilusApplication *application,
-							 GdkScreen           *screen);
+NautilusWindow *     nautilus_application_create_window (NautilusApplication *application);
 
 void nautilus_application_set_accelerator (GApplication *app,
 					   const gchar  *action_name,
@@ -86,4 +83,5 @@ GtkWidget * nautilus_application_connect_server (NautilusApplication *applicatio
 void nautilus_application_search (NautilusApplication *application,
                                   NautilusQuery       *query);
 void nautilus_application_startup_common (NautilusApplication *application);
+gboolean nautilus_application_is_sandboxed (void);
 G_END_DECLS
