@@ -27,6 +27,7 @@
 #include <time.h>
 #include <errno.h>
 #include <gtk/gtk.h>
+#include <libadwaita-1/adwaita.h>
 #include <gio/gio.h>
 
 #include <glib/gi18n.h>
@@ -47,7 +48,7 @@ autorun_software_dialog_destroy (AutorunSoftwareDialogData *data)
                                           G_CALLBACK (autorun_software_dialog_mount_unmounted),
                                           data);
 
-    gtk_widget_destroy (GTK_WIDGET (data->dialog));
+    gtk_window_destroy (GTK_WINDOW (data->dialog));
     g_object_unref (data->mount);
     g_free (data);
 }
@@ -116,7 +117,7 @@ autorun (GMount *mount)
     {
         program_to_spawn = g_file_get_child (root, "autorun");
     }
-    else if (_check_file (root, "autorun.sh", TRUE))
+    else if (_check_file (root, "autorun.sh", FALSE))
     {
         program_to_spawn = g_file_new_for_path ("/bin/sh");
         program_parameter_file = g_file_get_child (root, "autorun.sh");
@@ -150,34 +151,22 @@ out:
     if (error_string != NULL)
     {
         GtkWidget *dialog;
-        dialog = gtk_message_dialog_new_with_markup (NULL,         /* TODO: parent window? */
-                                                     0,
-                                                     GTK_MESSAGE_ERROR,
-                                                     GTK_BUTTONS_OK,
-                                                     _("Oops! There was a problem running this software."));
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error_string);
-        /* This is required because we don't show dialogs in the
-         *  window picker and if the window pops under another window
-         *  there is no way to get it back. */
-        gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
+        dialog = adw_message_dialog_new (NULL,
+                                         _("Oops! There was a problem running this software."),
+                                         error_string);
+        adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "ok", _("_OK"));
+        adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "ok");
 
-        g_signal_connect (dialog,
-                          "response",
-                          G_CALLBACK (gtk_widget_destroy),
-                          NULL);
-
-        gtk_widget_show_all (dialog);
+        gtk_window_present (GTK_WINDOW (dialog));
     }
 }
 
 static void
 autorun_software_dialog_response (GtkDialog *dialog,
-                                  gint       response_id,
+                                  gchar     *response,
                                   GMount    *mount)
 {
-    gtk_widget_destroy (GTK_WIDGET (dialog));
-
-    if (response_id == GTK_RESPONSE_OK)
+    if (g_strcmp0 (response, "run") == 0)
     {
         autorun (mount);
     }
@@ -193,20 +182,10 @@ present_autorun_for_software_dialog (GMount *mount)
 
     mount_name = g_mount_get_name (mount);
 
-    dialog = gtk_message_dialog_new (NULL,     /* TODO: parent window? */
-                                     0,
-                                     GTK_MESSAGE_OTHER,
-                                     GTK_BUTTONS_CANCEL,
-                                     _("“%s” contains software intended to be automatically started. Would you like to run it?"),
-                                     mount_name);
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                              "%s",
-                                              _("If you don’t trust this location or aren’t sure, press Cancel."));
-
-    /* This is required because we don't show dialogs in the
-     *  window picker and if the window pops under another window
-     *  there is no way to get it back. */
-    gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
+    dialog = adw_message_dialog_new (NULL, NULL, _("If you don’t trust this location or aren’t sure, press Cancel."));
+    adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog),
+                                       _("“%s” contains software intended to be automatically started. Would you like to run it?"),
+                                       mount_name);
 
     /* TODO: in a star trek future add support for verifying
      * software on media (e.g. if it has a certificate, check it
@@ -236,16 +215,18 @@ present_autorun_for_software_dialog (GMount *mount)
                       G_CALLBACK (autorun_software_dialog_mount_unmounted),
                       data);
 
-    gtk_dialog_add_button (GTK_DIALOG (dialog),
-                           _("_Run"),
-                           GTK_RESPONSE_OK);
+    adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+                                      "cancel", _("_Cancel"),
+                                      "run", _("_Run"),
+                                      NULL);
+    adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
 
     g_signal_connect (dialog,
                       "response",
                       G_CALLBACK (autorun_software_dialog_response),
                       mount);
 
-    gtk_widget_show_all (dialog);
+    gtk_window_present (GTK_WINDOW (dialog));
 }
 
 int
@@ -261,7 +242,8 @@ main (int   argc,
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
 
-    gtk_init (&argc, &argv);
+    gtk_init ();
+    adw_init ();
 
     if (argc != 2)
     {
@@ -293,6 +275,11 @@ main (int   argc,
 
     present_autorun_for_software_dialog (mount);
 
+    while (g_list_model_get_n_items (gtk_window_get_toplevels ()) > 0)
+    {
+        g_main_context_iteration (NULL, TRUE);
+    }
+
 out:
     return 0;
 }
