@@ -49,7 +49,7 @@ struct _NautilusBatchRenameDialog
     GtkWidget *replace_entry;
     GtkWidget *format_mode_button;
     GtkWidget *replace_mode_button;
-    GtkWidget *numbering_order_label;
+    GtkWidget *numbering_order_button;
     GtkWidget *numbering_label;
     GtkWidget *scrolled_window;
     GtkWidget *numbering_revealer;
@@ -63,8 +63,6 @@ struct _NautilusBatchRenameDialog
     GList *listbox_icons;
     GtkSizeGroup *size_group;
 
-    GList *motion_controllers;
-
     GList *selection;
     GList *new_names;
     NautilusBatchRenameDialogMode mode;
@@ -133,8 +131,8 @@ change_numbering_order (GSimpleAction *action,
     {
         if (g_strcmp0 (sorts_constants[i].action_target_name, target_name) == 0)
         {
-            gtk_label_set_label (GTK_LABEL (dialog->numbering_order_label),
-                                 gettext (sorts_constants[i].label));
+            gtk_menu_button_set_label (GTK_MENU_BUTTON (dialog->numbering_order_button),
+                                       gettext (sorts_constants[i].label));
             dialog->selection = nautilus_batch_rename_dialog_sort (dialog->selection,
                                                                    sorts_constants[i].sort_mode,
                                                                    dialog->create_date);
@@ -266,54 +264,6 @@ const GActionEntry dialog_entries[] =
     { "add-album-name-tag", add_metadata_tag },
 };
 
-static void
-row_selected (GtkListBox    *box,
-              GtkListBoxRow *listbox_row,
-              gpointer       user_data)
-{
-    NautilusBatchRenameDialog *dialog;
-    GtkListBoxRow *row;
-    gint index;
-
-    if (!GTK_IS_LIST_BOX_ROW (listbox_row))
-    {
-        return;
-    }
-
-    dialog = NAUTILUS_BATCH_RENAME_DIALOG (user_data);
-    index = gtk_list_box_row_get_index (listbox_row);
-
-    if (GTK_WIDGET (box) == dialog->original_name_listbox)
-    {
-        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->arrow_listbox), index);
-        gtk_list_box_select_row (GTK_LIST_BOX (dialog->arrow_listbox),
-                                 row);
-        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->result_listbox), index);
-        gtk_list_box_select_row (GTK_LIST_BOX (dialog->result_listbox),
-                                 row);
-    }
-
-    if (GTK_WIDGET (box) == dialog->arrow_listbox)
-    {
-        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->original_name_listbox), index);
-        gtk_list_box_select_row (GTK_LIST_BOX (dialog->original_name_listbox),
-                                 row);
-        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->result_listbox), index);
-        gtk_list_box_select_row (GTK_LIST_BOX (dialog->result_listbox),
-                                 row);
-    }
-
-    if (GTK_WIDGET (box) == dialog->result_listbox)
-    {
-        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->arrow_listbox), index);
-        gtk_list_box_select_row (GTK_LIST_BOX (dialog->arrow_listbox),
-                                 row);
-        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (dialog->original_name_listbox), index);
-        gtk_list_box_select_row (GTK_LIST_BOX (dialog->original_name_listbox),
-                                 row);
-    }
-}
-
 static gint
 compare_int (gconstpointer a,
              gconstpointer b)
@@ -429,14 +379,14 @@ batch_rename_dialog_get_new_names (NautilusBatchRenameDialog *dialog)
 
     if (dialog->mode == NAUTILUS_BATCH_RENAME_DIALOG_REPLACE)
     {
-        entry_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->find_entry)));
+        entry_text = g_strdup (gtk_editable_get_text (GTK_EDITABLE (dialog->find_entry)));
     }
     else
     {
-        entry_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->name_entry)));
+        entry_text = g_strdup (gtk_editable_get_text (GTK_EDITABLE (dialog->name_entry)));
     }
 
-    replace_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->replace_entry)));
+    replace_text = g_strdup (gtk_editable_get_text (GTK_EDITABLE (dialog->replace_entry)));
 
     if (dialog->mode == NAUTILUS_BATCH_RENAME_DIALOG_REPLACE)
     {
@@ -474,7 +424,7 @@ begin_batch_rename (NautilusBatchRenameDialog *dialog,
     /* do the actual rename here */
     nautilus_file_batch_rename (dialog->selection, new_names, NULL, NULL);
 
-    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (dialog->window)), NULL);
+    gtk_widget_set_cursor (GTK_WIDGET (dialog->window), NULL);
 }
 
 static void
@@ -584,7 +534,7 @@ create_original_name_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_labels_old = g_list_prepend (dialog->listbox_labels_old, label_old);
 
-    gtk_widget_show_all (label_old);
+    gtk_widget_show (label_old);
 
     return label_old;
 }
@@ -603,7 +553,7 @@ create_result_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_labels_new = g_list_prepend (dialog->listbox_labels_new, label_new);
 
-    gtk_widget_show_all (label_new);
+    gtk_widget_show (label_new);
 
     return label_new;
 }
@@ -629,7 +579,7 @@ create_arrow (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_icons = g_list_prepend (dialog->listbox_icons, icon);
 
-    gtk_widget_show_all (icon);
+    gtk_widget_show (icon);
 
     return icon;
 }
@@ -637,9 +587,6 @@ create_arrow (NautilusBatchRenameDialog *dialog,
 static void
 prepare_batch_rename (NautilusBatchRenameDialog *dialog)
 {
-    GdkCursor *cursor;
-    GdkDisplay *display;
-
     /* wait for checking conflicts to finish, to be sure that
      * the rename can actually take place */
     if (dialog->directories_pending_conflict_check != NULL)
@@ -653,22 +600,14 @@ prepare_batch_rename (NautilusBatchRenameDialog *dialog)
         return;
     }
 
-    display = gtk_widget_get_display (GTK_WIDGET (dialog->window));
-    cursor = gdk_cursor_new_from_name (display, "progress");
-    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (dialog->window)),
-                           cursor);
-    g_object_unref (cursor);
+    gtk_widget_set_cursor_from_name (GTK_WIDGET (dialog->window), "progress");
 
-    display = gtk_widget_get_display (GTK_WIDGET (dialog));
-    cursor = gdk_cursor_new_from_name (display, "progress");
-    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (dialog)),
-                           cursor);
-    g_object_unref (cursor);
+    gtk_widget_set_cursor_from_name (GTK_WIDGET (dialog), "progress");
 
     gtk_widget_hide (GTK_WIDGET (dialog));
     begin_batch_rename (dialog, dialog->new_names);
 
-    gtk_widget_destroy (GTK_WIDGET (dialog));
+    gtk_window_destroy (GTK_WINDOW (dialog));
 }
 
 static void
@@ -687,7 +626,7 @@ batch_rename_dialog_on_response (NautilusBatchRenameDialog *dialog,
             cancel_conflict_check (dialog);
         }
 
-        gtk_widget_destroy (GTK_WIDGET (dialog));
+        gtk_window_destroy (GTK_WINDOW (dialog));
     }
 }
 
@@ -939,7 +878,7 @@ update_listbox (NautilusBatchRenameDialog *dialog)
         else
         {
             new_name = batch_rename_replace_label_text (old_name,
-                                                        gtk_entry_get_text (GTK_ENTRY (dialog->find_entry)));
+                                                        gtk_editable_get_text (GTK_EDITABLE (dialog->find_entry)));
             gtk_label_set_markup (GTK_LABEL (label), new_name->str);
 
             g_string_free (new_name, TRUE);
@@ -1227,11 +1166,11 @@ have_unallowed_character (NautilusBatchRenameDialog *dialog)
 
     if (dialog->mode == NAUTILUS_BATCH_RENAME_DIALOG_FORMAT)
     {
-        entry_text = gtk_entry_get_text (GTK_ENTRY (dialog->name_entry));
+        entry_text = gtk_editable_get_text (GTK_EDITABLE (dialog->name_entry));
     }
     else
     {
-        entry_text = gtk_entry_get_text (GTK_ENTRY (dialog->replace_entry));
+        entry_text = gtk_editable_get_text (GTK_EDITABLE (dialog->replace_entry));
     }
 
     if (strstr (entry_text, "/") != NULL)
@@ -1386,7 +1325,7 @@ update_display_text (NautilusBatchRenameDialog *dialog)
 static void
 batch_rename_dialog_mode_changed (NautilusBatchRenameDialog *dialog)
 {
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->format_mode_button)))
+    if (gtk_check_button_get_active (GTK_CHECK_BUTTON (dialog->format_mode_button)))
     {
         gtk_stack_set_visible_child_name (GTK_STACK (dialog->mode_stack), "format");
 
@@ -1589,6 +1528,11 @@ nautilus_batch_rename_dialog_initialize_actions (NautilusBatchRenameDialog *dial
 
     check_metadata_for_selection (dialog, dialog->selection,
                                   dialog->metadata_cancellable);
+
+    /* Make sure that the state is initialized to name-ascending */
+    action = g_action_map_lookup_action (G_ACTION_MAP (dialog->action_group),
+                                         "numbering-order-changed");
+    g_action_change_state (action, g_variant_new_string ("name-ascending"));
 }
 
 static void
@@ -1859,7 +1803,6 @@ nautilus_batch_rename_dialog_finalize (GObject *object)
     nautilus_directory_list_free (dialog->distinct_parent_directories);
 
     g_object_unref (dialog->size_group);
-    g_clear_list (&dialog->motion_controllers, g_object_unref);
 
     g_hash_table_destroy (dialog->tag_info_table);
 
@@ -1894,7 +1837,7 @@ nautilus_batch_rename_dialog_class_init (NautilusBatchRenameDialogClass *klass)
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, mode_stack);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, replace_mode_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, format_mode_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_label);
+    gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, scrolled_window);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_order_menu);
     gtk_widget_class_bind_template_child (widget_class, NautilusBatchRenameDialog, numbering_revealer);
@@ -1910,8 +1853,6 @@ nautilus_batch_rename_dialog_class_init (NautilusBatchRenameDialogClass *klass)
     gtk_widget_class_bind_template_callback (widget_class, select_next_conflict_up);
     gtk_widget_class_bind_template_callback (widget_class, select_next_conflict_down);
     gtk_widget_class_bind_template_callback (widget_class, batch_rename_dialog_on_response);
-    gtk_widget_class_bind_template_callback (widget_class, on_insert_text);
-    gtk_widget_class_bind_template_callback (widget_class, on_delete_text);
 }
 
 GtkWidget *
@@ -1994,7 +1935,7 @@ nautilus_batch_rename_dialog_new (GList             *selection,
 
     fill_display_listbox (dialog);
 
-    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), NULL);
+    gtk_widget_set_cursor (GTK_WIDGET (window), NULL);
 
     g_string_free (dialog_title, TRUE);
 
@@ -2007,15 +1948,13 @@ connect_to_pointer_motion_events (NautilusBatchRenameDialog *self,
 {
     GtkEventController *controller;
 
-    controller = gtk_event_controller_motion_new (listbox);
+    controller = gtk_event_controller_motion_new ();
+    gtk_widget_add_controller (listbox, controller);
     gtk_event_controller_set_propagation_phase (controller, GTK_PHASE_CAPTURE);
     g_signal_connect (controller, "leave",
                       G_CALLBACK (on_event_controller_motion_leave), self);
     g_signal_connect (controller, "motion",
                       G_CALLBACK (on_event_controller_motion_motion), self);
-
-    self->motion_controllers = g_list_prepend (self->motion_controllers,
-                                               controller);
 }
 
 static void
@@ -2086,9 +2025,10 @@ nautilus_batch_rename_dialog_init (NautilusBatchRenameDialog *self)
 
     self->row_height = -1;
 
-    g_signal_connect (self->original_name_listbox, "row-selected", G_CALLBACK (row_selected), self);
-    g_signal_connect (self->arrow_listbox, "row-selected", G_CALLBACK (row_selected), self);
-    g_signal_connect (self->result_listbox, "row-selected", G_CALLBACK (row_selected), self);
+    g_signal_connect_object (gtk_editable_get_delegate (GTK_EDITABLE (self->name_entry)),
+                             "delete-text", G_CALLBACK (on_delete_text), self, 0);
+    g_signal_connect_object (gtk_editable_get_delegate (GTK_EDITABLE (self->name_entry)),
+                             "insert-text", G_CALLBACK (on_insert_text), self, 0);
 
     self->size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
