@@ -168,31 +168,6 @@ bookmark_file_changed_callback (NautilusFile     *file,
     }
 }
 
-static void
-apply_warning_emblem (GIcon    **base,
-                      gboolean   symbolic)
-{
-    GIcon *emblemed_icon;
-    g_autoptr (GIcon) warning = NULL;
-    g_autoptr (GEmblem) emblem = NULL;
-
-    if (symbolic)
-    {
-        warning = g_themed_icon_new ("dialog-warning-symbolic");
-    }
-    else
-    {
-        warning = g_themed_icon_new ("dialog-warning");
-    }
-
-    emblem = g_emblem_new (warning);
-    emblemed_icon = g_emblemed_icon_new (*base, emblem);
-
-    g_object_unref (*base);
-
-    *base = emblemed_icon;
-}
-
 gboolean
 nautilus_bookmark_get_is_builtin (NautilusBookmark *bookmark)
 {
@@ -298,7 +273,13 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
     g_autoptr (GIcon) icon = NULL;
     g_autoptr (GIcon) symbolic_icon = NULL;
 
-    if (g_file_is_native (bookmark->location))
+    if (!bookmark->exists)
+    {
+        DEBUG ("%s: file does not exist, set warning icon", nautilus_bookmark_get_name (bookmark));
+        symbolic_icon = g_themed_icon_new ("dialog-warning-symbolic");
+        icon = g_themed_icon_new ("dialog-warning");
+    }
+    else if (g_file_is_native (bookmark->location))
     {
         symbolic_icon = get_native_icon (bookmark, TRUE);
         icon = get_native_icon (bookmark, FALSE);
@@ -309,14 +290,6 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
         icon = g_themed_icon_new (NAUTILUS_ICON_FULLCOLOR_FOLDER_REMOTE);
     }
 
-    if (!bookmark->exists)
-    {
-        DEBUG ("%s: file does not exist, add emblem", nautilus_bookmark_get_name (bookmark));
-
-        apply_warning_emblem (&icon, FALSE);
-        apply_warning_emblem (&symbolic_icon, TRUE);
-    }
-
     DEBUG ("%s: setting icon to default", nautilus_bookmark_get_name (bookmark));
 
     g_object_set (bookmark,
