@@ -91,9 +91,6 @@ struct NautilusFileDetails
 	GdkPixbuf *thumbnail;
 	time_t thumbnail_mtime;
 
-	GdkPixbuf *scaled_thumbnail;
-	double thumbnail_scale;
-
 	GList *mime_list; /* If this is a directory, the list of MIME types in it. */
 
 	/* Info you might get from a link (.desktop, .directory or nautilus link) */
