@@ -43,70 +43,6 @@
 #define GET_ANCESTOR(obj) \
     GTK_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (obj), GTK_TYPE_WINDOW))
 
-void
-nautilus_files_view_handle_netscape_url_drop (NautilusFilesView *view,
-                                              const char        *encoded_url,
-                                              const char        *target_uri,
-                                              GdkDragAction      action)
-{
-    char *url;
-    char **bits;
-    GList *uri_list = NULL;
-    GFile *f;
-
-    f = g_file_new_for_uri (target_uri);
-
-    if (!g_file_is_native (f))
-    {
-        show_dialog (_("Drag and drop is not supported."),
-                     _("Drag and drop is only supported on local file systems."),
-                     GET_ANCESTOR (view),
-                     GTK_MESSAGE_WARNING);
-        g_object_unref (f);
-        return;
-    }
-
-    g_object_unref (f);
-
-    /* _NETSCAPE_URL_ works like this: $URL\n$TITLE */
-    bits = g_strsplit (encoded_url, "\n", 0);
-    switch (g_strv_length (bits))
-    {
-        case 0:
-        {
-            g_strfreev (bits);
-            return;
-        }
-
-        default:
-        {
-            url = bits[0];
-        }
-    }
-
-    f = g_file_new_for_uri (url);
-
-    if ((action != GDK_ACTION_COPY) && (action != GDK_ACTION_MOVE))
-    {
-        show_dialog (_("Drag and drop is not supported."),
-                     _("An invalid drag type was used."),
-                     GET_ANCESTOR (view),
-                     GTK_MESSAGE_WARNING);
-        return;
-    }
-
-    uri_list = g_list_append (uri_list, url);
-
-    nautilus_files_view_move_copy_items (view, uri_list,
-                                         target_uri,
-                                         action);
-
-    g_list_free (uri_list);
-
-    g_object_unref (f);
-    g_strfreev (bits);
-}
-
 void
 nautilus_files_view_handle_uri_list_drop (NautilusFilesView *view,
                                           const char        *item_uris,
@@ -133,10 +69,12 @@ nautilus_files_view_handle_uri_list_drop (NautilusFilesView *view,
 
     if (action == GDK_ACTION_ASK)
     {
+#if 0 && NAUTILUS_DND_NEEDS_GTK4_REIMPLEMENTATION
         action = nautilus_drag_drop_action_ask
                      (GTK_WIDGET (view),
                      GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
         if (action == 0)
+#endif
         {
             g_free (container_uri);
             return;
@@ -182,8 +120,8 @@ nautilus_files_view_handle_uri_list_drop (NautilusFilesView *view,
     g_free (container_uri);
 }
 
-#define MAX_LEN_FILENAME 128
-#define MIN_LEN_FILENAME 10
+#define MAX_LEN_FILENAME 64
+#define MIN_LEN_FILENAME 8
 
 static char *
 get_drop_filename (const char *text)
@@ -192,8 +130,9 @@ get_drop_filename (const char *text)
     char trimmed[MAX_LEN_FILENAME];
     int i;
     int last_word = -1;
-    int last_sentence = -1;
+    int end_sentence = -1;
     int last_nonspace = -1;
+    int start_sentence = -1;
     int num_attrs;
     PangoLogAttr *attrs;
     gchar *current_char;
@@ -206,35 +145,39 @@ get_drop_filename (const char *text)
     /* since the end of the text will always match a word boundary don't include it */
     for (i = 0; (i < num_attrs - 1); i++)
     {
+        if (attrs[i].is_sentence_start && start_sentence == -1)
+        {
+            start_sentence = i;
+        }
         if (!attrs[i].is_white)
         {
             last_nonspace = i;
         }
-        if (attrs[i].is_sentence_end)
-        {
-            last_sentence = last_nonspace;
-        }
         if (attrs[i].is_word_boundary)
         {
             last_word = last_nonspace;
         }
+        if (attrs[i].is_sentence_end)
+        {
+            end_sentence = last_nonspace;
+            break;
+        }
     }
     g_free (attrs);
 
-    if (last_sentence > 0)
+    if (end_sentence > 0)
     {
-        i = last_sentence;
+        i = end_sentence;
     }
     else
     {
         i = last_word;
     }
 
-    if (i > MIN_LEN_FILENAME)
+    if (i - start_sentence > MIN_LEN_FILENAME)
     {
-        char basename[MAX_LEN_FILENAME];
-        g_utf8_strncpy (basename, trimmed, i);
-        filename = g_strdup_printf ("%s.txt", basename);
+        g_autofree char *substring = g_utf8_substring (trimmed, start_sentence, i);
+        filename = g_strdup_printf ("%s.txt", substring);
     }
     else
     {
@@ -347,34 +290,44 @@ nautilus_files_view_drop_proxy_received_uris (NautilusFilesView *view,
                                               const char        *target_uri,
                                               GdkDragAction      action)
 {
-    char *container_uri;
+    g_autofree char *container_uri = NULL;
+    g_autoptr (GFile) source_location = g_file_new_for_uri (source_uri_list->data);
+    g_autoptr (GFile) target_location = g_file_new_for_uri (target_uri);
 
-    container_uri = NULL;
     if (target_uri == NULL)
     {
         container_uri = nautilus_files_view_get_backing_uri (view);
         g_assert (container_uri != NULL);
     }
+    if (g_file_has_parent (source_location, target_location) &&
+        action & GDK_ACTION_MOVE)
+    {
+        /* By default dragging to the same directory is allowed so that
+         * users can duplicate a file using the CTRL modifier key.  Prevent
+         * an accidental MOVE, by rejecting what would be an error anyways. */
+        return;
+    }
 
     if (action == GDK_ACTION_ASK)
     {
+#if 0 && NAUTILUS_DND_NEEDS_GTK4_REIMPLEMENTATION
         action = nautilus_drag_drop_action_ask
                      (GTK_WIDGET (view),
                      GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
         if (action == 0)
+#endif
         {
             return;
         }
     }
 
+#if 0 && NAUTILUS_DND_NEEDS_GTK4_REIMPLEMENTATION
     nautilus_clipboard_clear_if_colliding_uris (GTK_WIDGET (view),
                                                 source_uri_list);
-
+#endif
     nautilus_files_view_move_copy_items (view, source_uri_list,
                                          target_uri != NULL ? target_uri : container_uri,
                                          action);
-
-    g_free (container_uri);
 }
 
 void
