@@ -38,10 +38,10 @@ G_BEGIN_DECLS
 
 #define NAUTILUS_TYPE_FILES_VIEW nautilus_files_view_get_type()
 
-G_DECLARE_DERIVABLE_TYPE (NautilusFilesView, nautilus_files_view, NAUTILUS, FILES_VIEW, GtkGrid)
+G_DECLARE_DERIVABLE_TYPE (NautilusFilesView, nautilus_files_view, NAUTILUS, FILES_VIEW, AdwBin)
 
 struct _NautilusFilesViewClass {
-        GtkGridClass parent_class;
+        AdwBinClass parent_class;
 
         /* The 'clear' signal is emitted to empty the view of its contents.
          * It must be replaced by each subclass.
@@ -165,9 +165,6 @@ struct _NautilusFilesViewClass {
          * return whether the view is at minimum size (furthest-out zoom level) */
         gboolean (* can_zoom_out)            (NautilusFilesView *view);
 
-        /* The current zoom level as a percentage of the default. */
-        gfloat   (* get_zoom_level_percentage) (NautilusFilesView *view);
-
         gboolean (*is_zoom_level_default)      (NautilusFilesView *view);
 
         /* reveal_selection is a function pointer that subclasses may
@@ -186,28 +183,11 @@ struct _NautilusFilesViewClass {
 
         void    (* update_actions_state)     (NautilusFilesView *view);
 
-        /* sort_files is a function pointer that subclasses can override
-         * to provide a sorting order to determine which files should be
-         * presented when only a partial list is provided.
-         */
-        int     (* compare_files)            (NautilusFilesView *view,
-                                              NautilusFile      *a,
-                                              NautilusFile      *b);
-
         /* is_empty is a function pointer that subclasses must
          * override to report whether the view contains any items.
          */
         gboolean (* is_empty)                (NautilusFilesView *view);
 
-        /* convert *point from widget's coordinate system to a coordinate
-         * system used for specifying file operation positions, which is view-specific.
-         *
-         * This is used by the the icon view, which converts the screen position to a zoom
-         * level-independent coordinate system.
-         */
-        void (* widget_to_file_operation_position) (NautilusFilesView *view,
-                                                    GdkPoint     *position);
-
         /* Preference change callbacks, overridden by icon and list views.
          * Icon and list views respond by synchronizing to the new preference
          * values and forcing an update if appropriate.
@@ -231,8 +211,6 @@ struct _NautilusFilesViewClass {
 
         GdkRectangle * (* reveal_for_selection_context_menu) (NautilusFilesView *view);
 
-        GIcon *        (* get_icon)          (NautilusFilesView *view);
-
         /* Use this to show an optional visual feedback when the directory is empty.
          * By default it shows a widget overlay on top of the view */
         void           (* check_empty_states)          (NautilusFilesView *view);
@@ -266,15 +244,15 @@ void                nautilus_files_view_activate_files                   (Nautil
 void                nautilus_files_view_activate_file                    (NautilusFilesView *view,
                                                                           NautilusFile      *file,
                                                                           NautilusOpenFlags  flags);
-void                nautilus_files_view_start_batching_selection_changes (NautilusFilesView *view);
-void                nautilus_files_view_stop_batching_selection_changes  (NautilusFilesView *view);
 void                nautilus_files_view_notify_selection_changed         (NautilusFilesView *view);
 NautilusDirectory  *nautilus_files_view_get_model                        (NautilusFilesView *view);
 NautilusFile       *nautilus_files_view_get_directory_as_file            (NautilusFilesView *view);
 void                nautilus_files_view_pop_up_background_context_menu   (NautilusFilesView *view,
-                                                                          const GdkEvent    *event);
+                                                                          gdouble            x,
+                                                                          gdouble            y);
 void                nautilus_files_view_pop_up_selection_context_menu    (NautilusFilesView *view,
-                                                                          const GdkEvent    *event);
+                                                                          gdouble            x,
+                                                                          gdouble            y);
 gboolean            nautilus_files_view_should_show_file                 (NautilusFilesView *view,
                                                                           NautilusFile      *file);
 gboolean            nautilus_files_view_should_sort_directories_first    (NautilusFilesView *view);
@@ -300,6 +278,14 @@ void              nautilus_files_view_new_file_with_initial_contents (NautilusFi
                                                                       const char         *initial_contents,
                                                                       int                 length);
 
+/* clipboard reading */
+void               nautilus_files_view_get_clipboard_async  (NautilusFilesView   *self,
+                                                             GAsyncReadyCallback  callback,
+                                                             gpointer             callback_data);
+NautilusClipboard *nautilus_files_view_get_clipboard_finish (NautilusFilesView  *self,
+                                                             GAsyncResult       *result,
+                                                             GError            **error);
+
 /* selection handling */
 void              nautilus_files_view_activate_selection         (NautilusFilesView      *view);
 void              nautilus_files_view_preview_selection_event    (NautilusFilesView      *view,
