@@ -27,7 +27,6 @@
 #include "nautilus-file.h"
 #include "src/nautilus-files-view.h"
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-gtk-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
 #include <glib/gi18n.h>
@@ -39,7 +38,6 @@ GSettings *nautilus_list_view_preferences;
 GSettings *nautilus_window_state;
 GSettings *gtk_filechooser_preferences;
 GSettings *gnome_lockdown_preferences;
-GSettings *gnome_background_preferences;
 GSettings *gnome_interface_preferences;
 GSettings *gnome_privacy_preferences;
 
@@ -61,10 +59,9 @@ nautilus_global_preferences_init (void)
     nautilus_icon_view_preferences = g_settings_new ("org.gnome.nautilus.icon-view");
     nautilus_list_view_preferences = g_settings_new ("org.gnome.nautilus.list-view");
     /* Some settings such as show hidden files are shared between Nautilus and GTK file chooser */
-    gtk_filechooser_preferences = g_settings_new_with_path ("org.gtk.Settings.FileChooser",
-                                                            "/org/gtk/settings/file-chooser/");
+    gtk_filechooser_preferences = g_settings_new_with_path ("org.gtk.gtk4.Settings.FileChooser",
+                                                            "/org/gtk/gtk4/settings/file-chooser/");
     gnome_lockdown_preferences = g_settings_new ("org.gnome.desktop.lockdown");
-    gnome_background_preferences = g_settings_new ("org.gnome.desktop.background");
     gnome_interface_preferences = g_settings_new ("org.gnome.desktop.interface");
     gnome_privacy_preferences = g_settings_new ("org.gnome.desktop.privacy");
 }
