@@ -101,6 +101,7 @@ update_emblems (NautilusGridCell *self)
     g_autoptr (NautilusViewItem) item = NULL;
     NautilusFile *file;
     GtkWidget *child;
+    GtkIconTheme *theme;
     g_autolist (GIcon) emblems = NULL;
 
     item = nautilus_view_cell_get_item (NAUTILUS_VIEW_CELL (self));
@@ -113,9 +114,18 @@ update_emblems (NautilusGridCell *self)
         gtk_box_remove (GTK_BOX (self->emblems_box), child);
     }
 
+    theme = gtk_icon_theme_get_for_display (gdk_display_get_default ());
     emblems = nautilus_file_get_emblem_icons (file);
     for (GList *l = emblems; l != NULL; l = l->next)
     {
+        if (!gtk_icon_theme_has_gicon (theme, l->data))
+        {
+            g_autofree gchar *icon_string = g_icon_to_string (l->data);
+            g_warning ("Failed to add emblem. “%s” not found in the icon theme",
+                       icon_string);
+            continue;
+        }
+
         gtk_box_append (GTK_BOX (self->emblems_box),
                         gtk_image_new_from_gicon (l->data));
     }
@@ -169,6 +179,25 @@ on_item_is_cut_changed (NautilusGridCell *self)
     }
 }
 
+static gboolean
+on_label_query_tooltip (GtkWidget  *widget,
+                        int         x,
+                        int         y,
+                        gboolean    keyboard_tip,
+                        GtkTooltip *tooltip,
+                        gpointer    user_data)
+{
+    GtkLabel *label = GTK_LABEL (widget);
+
+    if (pango_layout_is_ellipsized (gtk_label_get_layout (label)))
+    {
+        gtk_tooltip_set_markup (tooltip, gtk_label_get_label (label));
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
 static void
 finalize (GObject *object)
 {
@@ -204,6 +233,9 @@ nautilus_grid_cell_init (NautilusGridCell *self)
 {
     gtk_widget_init_template (GTK_WIDGET (self));
 
+    g_signal_connect (self->label, "query-tooltip",
+                      G_CALLBACK (on_label_query_tooltip), NULL);
+
     /* Connect automatically to an item. */
     self->item_signal_group = g_signal_group_new (NAUTILUS_TYPE_VIEW_ITEM);
     g_signal_group_connect_swapped (self->item_signal_group, "notify::icon-size",
