@@ -26,7 +26,8 @@
 
 typedef enum {
         NAUTILUS_QUERY_SEARCH_TYPE_LAST_ACCESS,
-        NAUTILUS_QUERY_SEARCH_TYPE_LAST_MODIFIED
+        NAUTILUS_QUERY_SEARCH_TYPE_LAST_MODIFIED,
+        NAUTILUS_QUERY_SEARCH_TYPE_CREATED
 } NautilusQuerySearchType;
 
 typedef enum {
