@@ -147,6 +147,10 @@ model_directory_ready_cb (NautilusDirectory *directory,
 
     for (l = files; l != NULL; l = l->next)
     {
+        g_autoptr (GDateTime) mtime = NULL;
+        g_autoptr (GDateTime) atime = NULL;
+        g_autoptr (GDateTime) ctime = NULL;
+
         file = l->data;
 
         display_name = nautilus_file_get_display_name (file);
@@ -167,28 +171,49 @@ model_directory_ready_cb (NautilusDirectory *directory,
             }
         }
 
+        mtime = g_date_time_new_from_unix_local (nautilus_file_get_mtime (file));
+        atime = g_date_time_new_from_unix_local (nautilus_file_get_atime (file));
+        ctime = g_date_time_new_from_unix_local (nautilus_file_get_btime (file));
+
         date_range = nautilus_query_get_date_range (model->query);
         if (found && date_range != NULL)
         {
             NautilusQuerySearchType type;
-            guint64 current_file_unix_time;
+            GDateTime *target_date;
 
             type = nautilus_query_get_search_type (model->query);
             initial_date = g_ptr_array_index (date_range, 0);
             end_date = g_ptr_array_index (date_range, 1);
 
-            if (type == NAUTILUS_QUERY_SEARCH_TYPE_LAST_ACCESS)
+            switch (type)
             {
-                current_file_unix_time = nautilus_file_get_atime (file);
-            }
-            else
-            {
-                current_file_unix_time = nautilus_file_get_mtime (file);
+                case NAUTILUS_QUERY_SEARCH_TYPE_LAST_ACCESS:
+                {
+                    target_date = atime;
+                }
+                break;
+
+                case NAUTILUS_QUERY_SEARCH_TYPE_LAST_MODIFIED:
+                {
+                    target_date = mtime;
+                }
+                break;
+
+                case NAUTILUS_QUERY_SEARCH_TYPE_CREATED:
+                {
+                    target_date = ctime;
+                }
+                break;
+
+                default:
+                {
+                    target_date = NULL;
+                }
             }
 
-            found = nautilus_file_date_in_between (current_file_unix_time,
-                                                   initial_date,
-                                                   end_date);
+            found = nautilus_date_time_is_between_dates (target_date,
+                                                         initial_date,
+                                                         end_date);
             g_ptr_array_unref (date_range);
         }
 
@@ -197,6 +222,10 @@ model_directory_ready_cb (NautilusDirectory *directory,
             uri = nautilus_file_get_uri (file);
             hit = nautilus_search_hit_new (uri);
             nautilus_search_hit_set_fts_rank (hit, match);
+            nautilus_search_hit_set_modification_time (hit, mtime);
+            nautilus_search_hit_set_access_time (hit, atime);
+            nautilus_search_hit_set_creation_time (hit, ctime);
+
             hits = g_list_prepend (hits, hit);
 
             g_free (uri);
