@@ -179,6 +179,7 @@ cursor_callback (GObject      *object,
     const char *uri;
     const char *mtime_str;
     const char *atime_str;
+    const char *ctime_str;
     const gchar *snippet;
     GTimeVal tv;
     gdouble rank, match;
@@ -204,7 +205,8 @@ cursor_callback (GObject      *object,
     uri = tracker_sparql_cursor_get_string (cursor, 0, NULL);
     rank = tracker_sparql_cursor_get_double (cursor, 1);
     mtime_str = tracker_sparql_cursor_get_string (cursor, 2, NULL);
-    atime_str = tracker_sparql_cursor_get_string (cursor, 3, NULL);
+    ctime_str = tracker_sparql_cursor_get_string (cursor, 3, NULL);
+    atime_str = tracker_sparql_cursor_get_string (cursor, 4, NULL);
     basename = g_path_get_basename (uri);
 
     hit = nautilus_search_hit_new (uri);
@@ -214,8 +216,19 @@ cursor_callback (GObject      *object,
 
     if (tracker->fts_enabled)
     {
-        snippet = tracker_sparql_cursor_get_string (cursor, 4, NULL);
-        nautilus_search_hit_set_fts_snippet (hit, snippet);
+        snippet = tracker_sparql_cursor_get_string (cursor, 5, NULL);
+        if (snippet != NULL)
+        {
+            g_autofree gchar *escaped = NULL;
+            g_autoptr (GString) buffer = NULL;
+            /* Escape for markup, before adding our own markup. */
+            escaped = g_markup_escape_text (snippet, -1);
+            buffer = g_string_new (escaped);
+            g_string_replace (buffer, "_NAUTILUS_SNIPPET_DELIM_START_", "<b>", 0);
+            g_string_replace (buffer, "_NAUTILUS_SNIPPET_DELIM_END_", "</b>", 0);
+
+            nautilus_search_hit_set_fts_snippet (hit, buffer->str);
+        }
     }
 
     if (g_time_val_from_iso8601 (mtime_str, &tv))
@@ -240,6 +253,17 @@ cursor_callback (GObject      *object,
     {
         g_warning ("unable to parse atime: %s", atime_str);
     }
+    if (g_time_val_from_iso8601 (ctime_str, &tv))
+    {
+        GDateTime *date;
+        date = g_date_time_new_from_timeval_local (&tv);
+        nautilus_search_hit_set_creation_time (hit, date);
+        g_date_time_unref (date);
+    }
+    else
+    {
+        g_warning ("unable to parse ctime: %s", ctime_str);
+    }
 
     g_queue_push_head (tracker->hits_pending, hit);
     check_pending_hits (tracker, FALSE);
@@ -339,11 +363,17 @@ nautilus_search_engine_tracker_start (NautilusSearchProvider *provider)
                            " ?url"
                            " xsd:double(COALESCE(?rank2, ?rank1)) AS ?rank"
                            " nfo:fileLastModified(?file)"
+                           " nfo:fileCreated(?file)"
                            " nfo:fileLastAccessed(?file)");
 
     if (tracker->fts_enabled && *search_text)
     {
-        g_string_append (sparql, " fts:snippet(?content)");
+        g_string_append (sparql,
+                         "fts:snippet(?content,"
+                         "            '_NAUTILUS_SNIPPET_DELIM_START_',"
+                         "            '_NAUTILUS_SNIPPET_DELIM_END_', "
+                         "            '…',"
+                         "            20)");
     }
 
     g_string_append (sparql, "FROM tracker:FileSystem ");
@@ -359,7 +389,8 @@ nautilus_search_engine_tracker_start (NautilusSearchProvider *provider)
                      "  nfo:fileLastModified ?mtime;"
                      "  nfo:fileLastAccessed ?atime;"
                      "  nie:dataSource/tracker:available true;"
-                     "  nie:url ?url.");
+                     "  nie:url ?url."
+                     "  OPTIONAL { ?file nfo:fileCreated ?ctime.}");
 
     if (mimetypes->len > 0)
     {
@@ -429,11 +460,16 @@ nautilus_search_engine_tracker_start (NautilusSearchProvider *provider)
             g_string_append_printf (sparql, "?atime >= \"%s\"^^xsd:dateTime", initial_date_format);
             g_string_append_printf (sparql, " && ?atime <= \"%s\"^^xsd:dateTime", end_date_format);
         }
-        else
+        else if (type == NAUTILUS_QUERY_SEARCH_TYPE_LAST_MODIFIED)
         {
             g_string_append_printf (sparql, "?mtime >= \"%s\"^^xsd:dateTime", initial_date_format);
             g_string_append_printf (sparql, " && ?mtime <= \"%s\"^^xsd:dateTime", end_date_format);
         }
+        else
+        {
+            g_string_append_printf (sparql, "?ctime >= \"%s\"^^xsd:dateTime", initial_date_format);
+            g_string_append_printf (sparql, " && ?ctime <= \"%s\"^^xsd:dateTime", end_date_format);
+        }
 
 
         g_free (initial_date_format);
