@@ -21,8 +21,7 @@
  *
  */
 
-#ifndef __NAUTILUS_SHELL_SEARCH_PROVIDER_H__
-#define __NAUTILUS_SHELL_SEARCH_PROVIDER_H__
+#pragma once
 
 #define NAUTILUS_TYPE_SHELL_SEARCH_PROVIDER nautilus_shell_search_provider_get_type()
 #define NAUTILUS_SHELL_SEARCH_PROVIDER(obj) \
@@ -37,6 +36,4 @@ NautilusShellSearchProvider * nautilus_shell_search_provider_new (void);
 gboolean nautilus_shell_search_provider_register   (NautilusShellSearchProvider *self,
                                                     GDBusConnection             *connection,
                                                     GError                     **error);
-void     nautilus_shell_search_provider_unregister (NautilusShellSearchProvider *self);
-
-#endif /* __NAUTILUS_SHELL_SEARCH_PROVIDER_H__ */
+void     nautilus_shell_search_provider_unregister (NautilusShellSearchProvider *self);
\ No newline at end of file
