@@ -49,6 +49,11 @@ struct _NautilusTagManager
 
 G_DEFINE_TYPE (NautilusTagManager, nautilus_tag_manager, G_TYPE_OBJECT);
 
+static NautilusTagManager *tag_manager = NULL;
+
+/* See nautilus_tag_manager_new_dummy() documentation for details. */
+static gboolean make_dummy_instance = FALSE;
+
 typedef struct
 {
     NautilusTagManager *tag_manager;
@@ -300,8 +305,6 @@ nautilus_tag_manager_query_starred_files (NautilusTagManager *self,
         return;
     }
 
-    self->cancellable = cancellable;
-
     tracker_sparql_statement_execute_async (self->query_starred_files,
                                             cancellable,
                                             on_get_starred_files_query_callback,
@@ -533,6 +536,8 @@ nautilus_tag_manager_finalize (GObject *object)
                                               self);
     }
 
+    g_cancellable_cancel (self->cancellable);
+    g_clear_object (&self->cancellable);
     g_clear_object (&self->notifier);
     g_clear_object (&self->db);
     g_clear_object (&self->query_file_is_starred);
@@ -566,20 +571,13 @@ nautilus_tag_manager_class_init (NautilusTagManagerClass *klass)
 }
 
 /**
- * nautilus_tag_manager_get:
- *
- * Gets a reference to the tag manager.
- *
- * If used to initialize a struct field, make sure to release on finalization.
- * If used to initialize a local variable, make sure to use g_autoptr().
+ * nautilus_tag_manager_new:
  *
  * Returns: (transfer full): the #NautilusTagManager singleton object.
  */
 NautilusTagManager *
-nautilus_tag_manager_get (void)
+nautilus_tag_manager_new (void)
 {
-    static NautilusTagManager *tag_manager = NULL;
-
     if (tag_manager != NULL)
     {
         return g_object_ref (tag_manager);
@@ -591,6 +589,34 @@ nautilus_tag_manager_get (void)
     return tag_manager;
 }
 
+/**
+ * nautilus_tag_manager_new_dummy:
+ *
+ * Creates a dummy tag manager without database.
+ *
+ * Useful only for tests where the tag manager is needed but not being tested
+ * and we don't want to fail the tests due to irrelevant D-Bus failures.
+ *
+ * Returns: (transfer full): the #NautilusTagManager singleton object.
+ */
+NautilusTagManager *
+nautilus_tag_manager_new_dummy (void)
+{
+    make_dummy_instance = TRUE;
+    return nautilus_tag_manager_new ();
+}
+
+/**
+ * nautilus_tag_manager_get:
+ *
+ * Returns: (transfer none): the #NautilusTagManager singleton object.
+ */
+NautilusTagManager *
+nautilus_tag_manager_get (void)
+{
+    return tag_manager;
+}
+
 static gboolean
 setup_database (NautilusTagManager  *self,
                 GCancellable        *cancellable,
@@ -647,15 +673,26 @@ setup_database (NautilusTagManager  *self,
     return TRUE;
 }
 
-/* Initialize the tag mananger. */
-void
-nautilus_tag_manager_set_cancellable (NautilusTagManager *self,
-                                      GCancellable       *cancellable)
+static void
+nautilus_tag_manager_init (NautilusTagManager *self)
 {
     g_autoptr (GError) error = NULL;
 
-    self->database_ok = setup_database (self, cancellable, &error);
+    self->starred_file_uris = g_hash_table_new_full (g_str_hash,
+                                                     g_str_equal,
+                                                     (GDestroyNotify) g_free,
+                                                     /* values are keys */
+                                                     NULL);
+    self->home = g_file_new_for_path (g_get_home_dir ());
 
+    if (make_dummy_instance)
+    {
+        /* Skip database initiation for nautilus_tag_manager_new_dummy(). */
+        return;
+    }
+
+    self->cancellable = g_cancellable_new ();
+    self->database_ok = setup_database (self, self->cancellable, &error);
     if (error)
     {
         g_warning ("Unable to initialize tag manager: %s", error->message);
@@ -664,7 +701,7 @@ nautilus_tag_manager_set_cancellable (NautilusTagManager *self,
 
     self->notifier = tracker_sparql_connection_create_notifier (self->db);
 
-    nautilus_tag_manager_query_starred_files (self, cancellable);
+    nautilus_tag_manager_query_starred_files (self, self->cancellable);
 
     g_signal_connect (self->notifier,
                       "events",
@@ -672,26 +709,15 @@ nautilus_tag_manager_set_cancellable (NautilusTagManager *self,
                       self);
 }
 
-static void
-nautilus_tag_manager_init (NautilusTagManager *self)
-{
-    self->starred_file_uris = g_hash_table_new_full (g_str_hash,
-                                                     g_str_equal,
-                                                     (GDestroyNotify) g_free,
-                                                     /* values are keys */
-                                                     NULL);
-    self->home = g_file_new_for_path (g_get_home_dir ());
-}
-
 gboolean
-nautilus_tag_manager_can_star_contents (NautilusTagManager *tag_manager,
+nautilus_tag_manager_can_star_contents (NautilusTagManager *self,
                                         GFile              *directory)
 {
     /* We only allow files to be starred inside the home directory for now.
      * This avoids the starred files database growing too big.
      * See https://gitlab.gnome.org/GNOME/nautilus/-/merge_requests/553#note_903108
      */
-    return g_file_has_prefix (directory, tag_manager->home) || g_file_equal (directory, tag_manager->home);
+    return g_file_has_prefix (directory, self->home) || g_file_equal (directory, self->home);
 }
 
 static void
@@ -713,7 +739,6 @@ update_moved_uris_callback (GObject      *object,
     else
     {
         g_autolist (NautilusFile) updated_files = NULL;
-        g_autoptr (NautilusTagManager) tag_manager = nautilus_tag_manager_get ();
 
         for (guint i = 0; i < new_uris->len; i++)
         {
