@@ -137,7 +137,32 @@ get_thumbnail_factory (void)
 
     if (thumbnail_factory == NULL)
     {
-        thumbnail_factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+        GdkDisplay *display = gdk_display_get_default ();
+        GListModel *monitors = gdk_display_get_monitors (display);
+        gint max_scale = 1;
+        GnomeDesktopThumbnailSize size;
+
+        for (guint i = 0; i < g_list_model_get_n_items (monitors); i++)
+        {
+            g_autoptr (GdkMonitor) monitor = g_list_model_get_item (monitors, i);
+
+            max_scale = MAX (max_scale, gdk_monitor_get_scale_factor (monitor));
+        }
+
+        if (max_scale <= 1)
+        {
+            size = GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE;
+        }
+        else if (max_scale <= 2)
+        {
+            size = GNOME_DESKTOP_THUMBNAIL_SIZE_XLARGE;
+        }
+        else
+        {
+            size = GNOME_DESKTOP_THUMBNAIL_SIZE_XXLARGE;
+        }
+
+        thumbnail_factory = gnome_desktop_thumbnail_factory_new (size);
     }
 
     return thumbnail_factory;
@@ -398,7 +423,7 @@ nautilus_create_thumbnail (NautilusFile *file)
     {
         /* Add the thumbnail to the list. */
         DEBUG ("(Main Thread) Adding thumbnail: %s\n",
-                 info->image_uri);
+               info->image_uri);
         g_queue_push_tail ((GQueue *) &thumbnails_to_make, info);
         node = g_queue_peek_tail_link ((GQueue *) &thumbnails_to_make);
         g_hash_table_insert (thumbnails_to_make_hash,
@@ -417,7 +442,7 @@ nautilus_create_thumbnail (NautilusFile *file)
     else
     {
         DEBUG ("(Main Thread) Updating non-current mtime: %s\n",
-                 info->image_uri);
+               info->image_uri);
 
         /* The file in the queue might need a new original mtime */
         existing_info = existing->data;
@@ -447,12 +472,13 @@ thumbnail_thread_func (GTask        *task,
     time_t current_orig_mtime = 0;
     time_t current_time;
     GList *node;
+    GError *error = NULL;
 
     thumbnail_factory = get_thumbnail_factory ();
 
     /* We loop until there are no more thumbails to make, at which point
      *  we exit the thread. */
-    for (;; )
+    for (;;)
     {
         DEBUG ("(Thumbnail Thread) Locking mutex\n");
 
@@ -515,7 +541,7 @@ thumbnail_thread_func (GTask        *task,
             current_time >= current_orig_mtime)
         {
             DEBUG ("(Thumbnail Thread) Skipping: %s\n",
-                     info->image_uri);
+                   info->image_uri);
 
             /* Reschedule thumbnailing via a change notification */
             g_timeout_add_seconds (1, thumbnail_thread_notify_file_changed,
@@ -525,31 +551,43 @@ thumbnail_thread_func (GTask        *task,
 
         /* Create the thumbnail. */
         DEBUG ("(Thumbnail Thread) Creating thumbnail: %s\n",
-                 info->image_uri);
+               info->image_uri);
 
         pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (thumbnail_factory,
                                                                      info->image_uri,
-                                                                     info->mime_type);
+                                                                     info->mime_type,
+                                                                     NULL,
+                                                                     &error);
 
         if (pixbuf)
         {
             DEBUG ("(Thumbnail Thread) Saving thumbnail: %s\n",
-                     info->image_uri);
+                   info->image_uri);
 
             gnome_desktop_thumbnail_factory_save_thumbnail (thumbnail_factory,
                                                             pixbuf,
                                                             info->image_uri,
-                                                            current_orig_mtime);
+                                                            current_orig_mtime,
+                                                            NULL,
+                                                            &error);
+            if (error)
+            {
+                DEBUG ("(Thumbnail Thread) Saving thumbnail failed: %s (%s)\n",
+                       info->image_uri, error->message);
+                g_clear_error (&error);
+            }
             g_object_unref (pixbuf);
         }
         else
         {
-            DEBUG ("(Thumbnail Thread) Thumbnail failed: %s\n",
-                     info->image_uri);
+            DEBUG ("(Thumbnail Thread) Thumbnail failed: %s (%s)\n",
+                   info->image_uri, error->message);
+            g_clear_error (&error);
 
             gnome_desktop_thumbnail_factory_create_failed_thumbnail (thumbnail_factory,
                                                                      info->image_uri,
-                                                                     current_orig_mtime);
+                                                                     current_orig_mtime,
+                                                                     NULL, NULL);
         }
         /* We need to call nautilus_file_changed(), but I don't think that is
          *  thread safe. So add an idle handler and do it from the main loop. */
