@@ -28,7 +28,6 @@
 
 #include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <libgd/gd.h>
 #include <string.h>
 #include <glib/gi18n.h>
 
@@ -68,28 +67,81 @@ nautilus_gmenu_set_from_model (GMenu      *target_menu,
     }
 }
 
-#define NAUTILUS_THUMBNAIL_FRAME_LEFT 3
-#define NAUTILUS_THUMBNAIL_FRAME_TOP 3
-#define NAUTILUS_THUMBNAIL_FRAME_RIGHT 3
-#define NAUTILUS_THUMBNAIL_FRAME_BOTTOM 3
-
-void
-nautilus_ui_frame_image (GdkPixbuf **pixbuf)
+/**
+ * nautilus_g_menu_model_find_by_string:
+ * @model: the #GMenuModel with items to search
+ * @attribute: the menu item attribute to compare with
+ * @string: the string to match the value of @attribute
+ *
+ * This will search for an item in the model which has got the @attribute and
+ * whose value is equal to @string.
+ *
+ * It is assumed that @attribute has the a GVariant format string "s".
+ *
+ * Returns: The index of the first match in the model, or -1 if no item matches.
+ */
+gint
+nautilus_g_menu_model_find_by_string (GMenuModel  *model,
+                                      const gchar *attribute,
+                                      const gchar *string)
 {
-    GtkBorder border;
-    GdkPixbuf *pixbuf_with_frame;
+    gint item_index = -1;
+    gint n_items;
 
-    border.left = NAUTILUS_THUMBNAIL_FRAME_LEFT;
-    border.top = NAUTILUS_THUMBNAIL_FRAME_TOP;
-    border.right = NAUTILUS_THUMBNAIL_FRAME_RIGHT;
-    border.bottom = NAUTILUS_THUMBNAIL_FRAME_BOTTOM;
+    n_items = g_menu_model_get_n_items (model);
+    for (gint i = 0; i < n_items; i++)
+    {
+        g_autofree gchar *value = NULL;
+        if (g_menu_model_get_item_attribute (model, i, attribute, "s", &value) &&
+            g_strcmp0 (value, string) == 0)
+        {
+            item_index = i;
+            break;
+        }
+    }
+    return item_index;
+}
 
-    pixbuf_with_frame = gd_embed_image_in_frame (*pixbuf,
-                                                 "resource:///org/gnome/nautilus/icons/thumbnail_frame.png",
-                                                 &border, &border);
-    g_object_unref (*pixbuf);
+/**
+ * nautilus_g_menu_replace_string_in_item:
+ * @menu: the #GMenu to modify
+ * @i: the position of the item to change
+ * @attribute: the menu item attribute to change
+ * @string: the string to change the value of @attribute to
+ *
+ * This will replace the item at @position with a new item which is identical
+ * except that it has @attribute set to @string.
+ *
+ * This is useful e.g. when want to change the menu model of a #GtkPopover and
+ * you have a pointer to its menu model but not to the popover itself, so you
+ * can't just set a new model. With this method, the GtkPopover is notified of
+ * changes in its model and updates its contents accordingly.
+ *
+ * It is assumed that @attribute has the a GVariant format string "s".
+ */
+void
+nautilus_g_menu_replace_string_in_item (GMenu       *menu,
+                                        gint         i,
+                                        const gchar *attribute,
+                                        const gchar *string)
+{
+    g_autoptr (GMenuItem) item = NULL;
 
-    *pixbuf = pixbuf_with_frame;
+    g_return_if_fail (i != -1);
+    item = g_menu_item_new_from_model (G_MENU_MODEL (menu), i);
+    g_return_if_fail (item != NULL);
+
+    if (string != NULL)
+    {
+        g_menu_item_set_attribute (item, attribute, "s", string);
+    }
+    else
+    {
+        g_menu_item_set_attribute (item, attribute, NULL);
+    }
+
+    g_menu_remove (menu, i);
+    g_menu_insert_item (menu, i, item);
 }
 
 static GdkPixbuf *filmholes_left = NULL;
@@ -112,57 +164,56 @@ ensure_filmholes (void)
 }
 
 void
-nautilus_ui_frame_video (GdkPixbuf **pixbuf)
+nautilus_ui_frame_video (GtkSnapshot *snapshot,
+                         gdouble      width,
+                         gdouble      height)
 {
-    int width, height;
+    g_autoptr (GdkTexture) left_texture = NULL;
+    g_autoptr (GdkTexture) right_texture = NULL;
     int holes_width, holes_height;
-    int i;
 
     if (!ensure_filmholes ())
     {
         return;
     }
 
-    width = gdk_pixbuf_get_width (*pixbuf);
-    height = gdk_pixbuf_get_height (*pixbuf);
     holes_width = gdk_pixbuf_get_width (filmholes_left);
     holes_height = gdk_pixbuf_get_height (filmholes_left);
 
-    for (i = 0; i < height; i += holes_height)
-    {
-        gdk_pixbuf_composite (filmholes_left, *pixbuf, 0, i,
-                              MIN (width, holes_width),
-                              MIN (height - i, holes_height),
-                              0, i, 1, 1, GDK_INTERP_NEAREST, 255);
-    }
+    /* Left */
+    gtk_snapshot_push_repeat (snapshot,
+                              &GRAPHENE_RECT_INIT (0, 0, holes_width, height),
+                              NULL);
+    left_texture = gdk_texture_new_for_pixbuf (filmholes_left);
+    gtk_snapshot_append_texture (snapshot,
+                                 left_texture,
+                                 &GRAPHENE_RECT_INIT (0, 0, holes_width, holes_height));
+    gtk_snapshot_pop (snapshot);
 
-    for (i = 0; i < height; i += holes_height)
-    {
-        gdk_pixbuf_composite (filmholes_right, *pixbuf,
-                              width - holes_width, i,
-                              MIN (width, holes_width),
-                              MIN (height - i, holes_height),
-                              width - holes_width, i,
-                              1, 1, GDK_INTERP_NEAREST, 255);
-    }
+    /* Right */
+    gtk_snapshot_push_repeat (snapshot,
+                              &GRAPHENE_RECT_INIT (width - holes_width, 0, holes_width, height),
+                              NULL);
+    right_texture = gdk_texture_new_for_pixbuf (filmholes_right);
+    gtk_snapshot_append_texture (snapshot,
+                                 right_texture,
+                                 &GRAPHENE_RECT_INIT (width - holes_width, 0, holes_width, holes_height));
+    gtk_snapshot_pop (snapshot);
 }
 
 gboolean
-nautilus_file_date_in_between (guint64    unix_file_time,
-                               GDateTime *initial_date,
-                               GDateTime *end_date)
+nautilus_date_time_is_between_dates (GDateTime *date,
+                                     GDateTime *initial_date,
+                                     GDateTime *end_date)
 {
-    GDateTime *date;
     gboolean in_between;
 
     /* Silently ignore errors */
-    if (unix_file_time == 0)
+    if (date == NULL || g_date_time_to_unix (date) == 0)
     {
         return FALSE;
     }
 
-    date = g_date_time_new_from_unix_local (unix_file_time);
-
     /* For the end date, we want to make end_date inclusive,
      * for that the difference between the start of the day and the in_between
      * has to be more than -1 day
@@ -170,8 +221,6 @@ nautilus_file_date_in_between (guint64    unix_file_time,
     in_between = g_date_time_difference (date, initial_date) > 0 &&
                  g_date_time_difference (end_date, date) / G_TIME_SPAN_DAY > -1;
 
-    g_date_time_unref (date);
-
     return in_between;
 }
 
@@ -268,7 +317,7 @@ get_text_for_date_range (GPtrArray *date_range,
     return label;
 }
 
-GtkDialog *
+AdwMessageDialog *
 show_dialog (const gchar    *primary_text,
              const gchar    *secondary_text,
              GtkWindow      *parent,
@@ -278,23 +327,13 @@ show_dialog (const gchar    *primary_text,
 
     g_return_val_if_fail (parent != NULL, NULL);
 
-    dialog = gtk_message_dialog_new (parent,
-                                     GTK_DIALOG_MODAL,
-                                     type,
-                                     GTK_BUTTONS_OK,
-                                     "%s", primary_text);
+    dialog = adw_message_dialog_new (parent, primary_text, secondary_text);
+    adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "ok", _("_OK"));
+    adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "ok");
 
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                              "%s", secondary_text);
+    gtk_window_present (GTK_WINDOW (dialog));
 
-    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-
-    gtk_widget_show (dialog);
-
-    g_signal_connect (GTK_DIALOG (dialog), "response",
-                      G_CALLBACK (gtk_widget_destroy), NULL);
-
-    return GTK_DIALOG (dialog);
+    return ADW_MESSAGE_DIALOG (dialog);
 }
 
 static void
