@@ -2,7 +2,7 @@
 
 #include <glib.h>
 #include "nautilus-file.h"
-#include "nautilus-view-item-model.h"
+#include "nautilus-view-item.h"
 
 G_BEGIN_DECLS
 
@@ -10,30 +10,25 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (NautilusViewModel, nautilus_view_model, NAUTILUS, VIEW_MODEL, GObject)
 
-typedef struct
-{
-    NautilusFileSortType sort_type;
-    gboolean reversed;
-    gboolean directories_first;
-} NautilusViewModelSortData;
-
 NautilusViewModel * nautilus_view_model_new (void);
 
-void nautilus_view_model_set_sort_type (NautilusViewModel         *self,
-                                        NautilusViewModelSortData *sort_data);
-NautilusViewModelSortData * nautilus_view_model_get_sort_type (NautilusViewModel *self);
-NautilusViewItemModel * nautilus_view_model_get_item_from_file (NautilusViewModel *self,
+GtkSorter *nautilus_view_model_get_sorter (NautilusViewModel *self);
+void nautilus_view_model_set_sorter (NautilusViewModel *self,
+                                     GtkSorter         *sorter);
+NautilusViewItem * nautilus_view_model_get_item_from_file (NautilusViewModel *self,
                                                                 NautilusFile      *file);
 GQueue * nautilus_view_model_get_items_from_files (NautilusViewModel *self,
                                                    GQueue            *files);
 /* Don't use inside a loop, use nautilus_view_model_remove_all_items instead. */
 void nautilus_view_model_remove_item (NautilusViewModel     *self,
-                                      NautilusViewItemModel *item);
+                                      NautilusViewItem *item);
 void nautilus_view_model_remove_all_items (NautilusViewModel *self);
 /* Don't use inside a loop, use nautilus_view_model_add_items instead. */
 void nautilus_view_model_add_item (NautilusViewModel     *self,
-                                   NautilusViewItemModel *item);
+                                   NautilusViewItem *item);
 void nautilus_view_model_add_items (NautilusViewModel *self,
                                     GQueue            *items);
+guint nautilus_view_model_get_index (NautilusViewModel     *self,
+                                     NautilusViewItem *item);
 
 G_END_DECLS
