@@ -27,14 +27,14 @@
 #pragma once
 
 #include <gtk/gtk.h>
-#include <libhandy-1/handy.h>
+#include <libadwaita-1/adwaita.h>
 
 #include "nautilus-types.h"
 
 G_BEGIN_DECLS
 
 #define NAUTILUS_TYPE_WINDOW (nautilus_window_get_type ())
-G_DECLARE_FINAL_TYPE (NautilusWindow, nautilus_window, NAUTILUS, WINDOW, HdyApplicationWindow);
+G_DECLARE_FINAL_TYPE (NautilusWindow, nautilus_window, NAUTILUS, WINDOW, AdwApplicationWindow);
 
 typedef gboolean (* NautilusWindowGoToCallback) (NautilusWindow *window,
                                                  GFile *location,
@@ -57,7 +57,14 @@ typedef void (* NautilusWindowHandleExported) (NautilusWindow *window,
 #define NAUTILUS_WINDOW_DEFAULT_WIDTH		890
 #define NAUTILUS_WINDOW_DEFAULT_HEIGHT		550
 
-NautilusWindow * nautilus_window_new                  (GdkScreen         *screen);
+typedef enum
+{
+    NAUTILUS_NAVIGATION_DIRECTION_NONE,
+    NAUTILUS_NAVIGATION_DIRECTION_BACK,
+    NAUTILUS_NAVIGATION_DIRECTION_FORWARD
+} NautilusNavigationDirection;
+
+NautilusWindow * nautilus_window_new                  (void);
 void             nautilus_window_close                (NautilusWindow    *window);
 
 void nautilus_window_open_location_full               (NautilusWindow     *window,
@@ -76,11 +83,9 @@ void                 nautilus_window_slot_close            (NautilusWindow *wind
 
 void                 nautilus_window_sync_location_widgets (NautilusWindow *window);
 
-void     nautilus_window_hide_sidebar         (NautilusWindow *window);
-void     nautilus_window_show_sidebar         (NautilusWindow *window);
 void nautilus_window_reset_menus (NautilusWindow *window);
 
-GtkWidget *         nautilus_window_get_notebook (NautilusWindow *window);
+AdwTabView * nautilus_window_get_tab_view (NautilusWindow *window);
 
 void     nautilus_window_show_about_dialog    (NautilusWindow *window);
 
@@ -97,10 +102,6 @@ void nautilus_window_sync_title            (NautilusWindow *window,
 void nautilus_window_show_operation_notification (NautilusWindow *window,
                                                   gchar          *main_label,
                                                   GFile          *folder_to_open);
-void nautilus_window_start_dnd (NautilusWindow *window,
-                                GdkDragContext *context);
-void nautilus_window_end_dnd (NautilusWindow *window,
-                              GdkDragContext *context);
 
 void nautilus_window_search (NautilusWindow *window,
                              NautilusQuery  *query);
@@ -114,4 +115,7 @@ gboolean nautilus_window_export_handle (NautilusWindow *window,
                                         gpointer user_data);
 void nautilus_window_unexport_handle (NautilusWindow *window);
 
+void nautilus_window_back_or_forward_in_new_tab (NautilusWindow              *window,
+                                                 NautilusNavigationDirection  back);
+
 G_END_DECLS
