@@ -30,11 +30,10 @@
 #include "nautilus-icon-info.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-program-choosing.h"
-#include "nautilus-gtk4-helpers.h"
 
 struct _NautilusXContentBar
 {
-    GtkInfoBar parent_instance;
+    AdwBin parent_instance;
     GtkWidget *label;
 
     char **x_content_types;
@@ -53,7 +52,7 @@ enum
     CONTENT_BAR_RESPONSE_APP = 1
 };
 
-G_DEFINE_TYPE (NautilusXContentBar, nautilus_x_content_bar, GTK_TYPE_INFO_BAR)
+G_DEFINE_TYPE (NautilusXContentBar, nautilus_x_content_bar, ADW_TYPE_BIN)
 
 static void
 content_bar_response_cb (GtkInfoBar *infobar,
@@ -79,7 +78,7 @@ content_bar_response_cb (GtkInfoBar *infobar,
     if (default_app != NULL)
     {
         nautilus_launch_application_for_mount (default_app, bar->mount,
-                                               GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (bar))));
+                                               GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (bar))));
         g_object_unref (default_app);
     }
 }
@@ -154,6 +153,7 @@ nautilus_x_content_bar_set_x_content_types (NautilusXContentBar *bar,
         const char *name;
         GIcon *icon;
         GtkWidget *image;
+        GtkWidget *info_bar;
         GtkWidget *button;
         GAppInfo *app;
         gboolean has_app;
@@ -181,7 +181,7 @@ nautilus_x_content_bar_set_x_content_types (NautilusXContentBar *bar,
         icon = g_app_info_get_icon (default_app);
         if (icon != NULL)
         {
-            image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_BUTTON);
+            image = gtk_image_new_from_gicon (icon);
         }
         else
         {
@@ -189,14 +189,15 @@ nautilus_x_content_bar_set_x_content_types (NautilusXContentBar *bar,
         }
 
         name = g_app_info_get_name (default_app);
-        button = gtk_info_bar_add_button (GTK_INFO_BAR (bar), name, n);
+        info_bar = adw_bin_get_child (ADW_BIN (bar));
+        button = gtk_info_bar_add_button (GTK_INFO_BAR (info_bar), name, n);
         box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
         if (image != NULL)
         {
-            gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
+            gtk_box_append (GTK_BOX (box), image);
         }
-        gtk_box_pack_start (GTK_BOX (box), gtk_label_new (name), FALSE, FALSE, 0);
+        gtk_box_append (GTK_BOX (box), gtk_label_new (name));
 
         gtk_button_set_child (GTK_BUTTON (button), box);
 
@@ -323,12 +324,13 @@ nautilus_x_content_bar_class_init (NautilusXContentBarClass *klass)
 static void
 nautilus_x_content_bar_init (NautilusXContentBar *bar)
 {
-    GtkWidget *action_area;
+    GtkWidget *info_bar;
     PangoAttrList *attrs;
 
-    action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (bar));
-
-    gtk_orientable_set_orientation (GTK_ORIENTABLE (action_area), GTK_ORIENTATION_HORIZONTAL);
+    info_bar = gtk_info_bar_new ();
+    gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_QUESTION);
+    gtk_widget_show (info_bar);
+    adw_bin_set_child (ADW_BIN (bar), info_bar);
 
     attrs = pango_attr_list_new ();
     pango_attr_list_insert (attrs, pango_attr_weight_new (PANGO_WEIGHT_BOLD));
@@ -337,9 +339,9 @@ nautilus_x_content_bar_init (NautilusXContentBar *bar)
     pango_attr_list_unref (attrs);
 
     gtk_label_set_ellipsize (GTK_LABEL (bar->label), PANGO_ELLIPSIZE_END);
-    gtk_info_bar_add_child (GTK_INFO_BAR (bar), bar->label);
+    gtk_info_bar_add_child (GTK_INFO_BAR (info_bar), bar->label);
 
-    g_signal_connect (bar, "response",
+    g_signal_connect (info_bar, "response",
                       G_CALLBACK (content_bar_response_cb),
                       bar);
 }
@@ -349,7 +351,6 @@ nautilus_x_content_bar_new (GMount             *mount,
                             const char * const *x_content_types)
 {
     return g_object_new (NAUTILUS_TYPE_X_CONTENT_BAR,
-                         "message-type", GTK_MESSAGE_QUESTION,
                          "mount", mount,
                          "x-content-types", x_content_types,
                          NULL);
